@@ -1,10 +1,10 @@
 /*******************************************************************************
  *
  * NSCA.C - Nagios Service Check Acceptor
- * Copyright (c) 2000-2003 Ethan Galstad (nagios@nagios.org)
- * License: GPL
+ * Copyright (c) 2000-2006 Ethan Galstad (nagios@nagios.org)
+ * License: GPL v2
  *
- * Last Modified: 07-23-2003
+ * Last Modified: 01-21-2006
  *
  * Command line: NSCA -c <config_file> [mode]
  *
@@ -15,13 +15,12 @@
  * 
  ******************************************************************************/
 
-#include "../common/common.h"
-#include "../common/config.h"
-#include "netutils.h"
-#include "utils.h"
+#include "../include/common.h"
+#include "../include/config.h"
+#include "../include/netutils.h"
+#include "../include/utils.h"
 
 
-static char allowed_hosts[MAX_INPUT_BUFFER];
 static int server_port=DEFAULT_SERVER_PORT;
 static char server_address[16]="0.0.0.0";
 static int socket_timeout=DEFAULT_SOCKET_TIMEOUT;
@@ -38,7 +37,6 @@ static void accept_connection(int,void *);
 static void handle_connection_read(int,void *);
 static int process_arguments(int,char **);
 static int read_config_file(char *);
-static int is_an_allowed_host(char *);
 static int open_command_file(void);
 static void close_command_file(void);
 static void install_child_handler(void);
@@ -78,6 +76,11 @@ int nrhand=0;
 int nwhand=0;
 int npfds=0;
 
+#ifdef HAVE_LIBWRAP
+int allow_severity=LOG_INFO;
+int deny_severity=LOG_WARNING;
+#endif
+
 
 
 int main(int argc, char **argv){
@@ -94,10 +97,10 @@ int main(int argc, char **argv){
 			printf("Incorrect command line arguments supplied\n");
                 printf("\n");
                 printf("NSCA - Nagios Service Check Acceptor\n");
-                printf("Copyright (c) 2000-2003 Ethan Galstad (nagios@nagios.org)\n");
+                printf("Copyright (c) 2000-2006 Ethan Galstad (www.nagios.org)\n");
                 printf("Version: %s\n",PROGRAM_VERSION);
                 printf("Last Modified: %s\n",MODIFICATION_DATE);
-                printf("License: GPL\n");
+                printf("License: GPL v2\n");
                 printf("Encryption Routines: ");
 #ifdef HAVE_LIBMCRYPT
                 printf("AVAILABLE");
@@ -105,6 +108,9 @@ int main(int argc, char **argv){
                 printf("NOT AVAILABLE");
 #endif          
                 printf("\n");
+#ifdef HAVE_LIBWRAP
+		printf("TCP Wrappers Available\n");
+#endif
                 printf("\n");
 	        }
 
@@ -221,12 +227,11 @@ int main(int argc, char **argv){
         }
 
 
-
-/* exit cleanly */
-static void do_exit(int return_code){
+/* cleanup */
+static void do_cleanup(void){
 
         /* close the command file if its still open */
-        if (command_file_fp!=NULL)
+        if(command_file_fp!=NULL)
                 close_command_file();
 
 	/*** CLEAR SENSITIVE INFO FROM MEMORY ***/
@@ -237,7 +242,16 @@ static void do_exit(int return_code){
 	/* disguise decryption method */
 	decryption_method=-1;
 
-        exit(return_code);
+	return;
+        }
+
+
+/* exit cleanly */
+static void do_exit(int return_code){
+
+	do_cleanup();
+
+	exit(return_code);
         }
 
 
@@ -297,14 +311,6 @@ static int read_config_file(char *filename){
                         strncpy(server_address,varvalue,sizeof(server_address) - 1);
                         server_address[sizeof(server_address)-1]='\0';
                         }
-		else if(!strcmp(varname,"allowed_hosts")){
-                        if(strlen(varvalue)>sizeof(allowed_hosts)-1){
-                                syslog(LOG_ERR,"Allowed hosts list too long in config file '%s' - Line %d\n",filename,line);
-                                return ERROR;
-                                }
-                        strncpy(allowed_hosts,varvalue,sizeof(allowed_hosts));
-                        allowed_hosts[sizeof(allowed_hosts)-1]='\0';
-                        }
 		else if(strstr(input_buffer,"command_file")){
                         if(strlen(varvalue)>sizeof(command_file)-1){
                                 syslog(LOG_ERR,"Command file name is too long in config file '%s' - Line %d\n",filename,line);
@@ -661,7 +667,6 @@ static void wait_for_connections(void) {
 
         if(debug==TRUE){
                 syslog(LOG_DEBUG,"Listening for connections on port %d\n",htons(myname.sin_port));
-                syslog(LOG_DEBUG,"Allowing connections from: %s\n",allowed_hosts);
                 }
 
         /* listen for connection requests */
@@ -685,9 +690,11 @@ static void accept_connection(int sock, void *unused){
         pid_t pid;
         struct sockaddr addr;
         struct sockaddr_in *nptr;
-        int addrlen;
+        socklen_t addrlen;
         int rc;
-        char connecting_host[16];
+#ifdef HAVE_LIBWRAP
+	struct request_info req;
+#endif
 
         if(mode==SINGLE_PROCESS_DAEMON)
                 register_read_handler(sock, accept_connection, NULL);
@@ -718,6 +725,21 @@ static void accept_connection(int sock, void *unused){
 		return;
                 }
 
+#ifdef HAVE_LIBWRAP
+
+	/* Check whether or not connections are allowed from this host */
+	request_init(&req,RQ_DAEMON,"nsca",RQ_FILE,new_sd,0);
+	fromhost(&req);
+
+	if(!hosts_access(&req)){
+		/* refuse the connection */
+		syslog(LOG_ERR, "refused connect from %s", eval_client(&req));
+		close(new_sd);
+		return;
+		}
+#endif
+
+
         /* fork() if we have to... */
         if(mode==MULTI_PROCESS_DAEMON){
 
@@ -754,32 +776,15 @@ static void accept_connection(int sock, void *unused){
         if(debug==TRUE)
                 syslog(LOG_DEBUG,"Connection from %s port %d",inet_ntoa(nptr->sin_addr),nptr->sin_port);
 
-        /* is this is blessed machine? */
-        snprintf(connecting_host,sizeof(connecting_host),"%s",inet_ntoa(nptr->sin_addr));
-        connecting_host[sizeof(connecting_host)-1]='\0';
+	/* handle the connection */
+	if(mode==SINGLE_PROCESS_DAEMON)
+		/* mark the connection as ready to be handled */
+		register_write_handler(new_sd, handle_connection, NULL);
+	else
+		/* handle the client connection */
+		handle_connection(new_sd, NULL);
 
-        if(!is_an_allowed_host(connecting_host)){
-
-                /* log error to syslog facility */
-                syslog(LOG_ERR,"Host %s is not allowed to talk to us!", inet_ntoa(nptr->sin_addr));
-		close(new_sd);
-		if(mode==MULTI_PROCESS_DAEMON)
-			do_exit(STATE_CRITICAL);
- 		return;
-	        }
-	else{
-
-                /* log info to syslog facility */
-                if(debug==TRUE)
-                        syslog(LOG_DEBUG,"Host address checks out ok");
-
-                if(mode==SINGLE_PROCESS_DAEMON)
-                        /* mark the connection as ready to be handled */
-                        register_write_handler(new_sd, handle_connection, NULL);
-                else
-                        /* handle the client connection */
-                        handle_connection(new_sd, NULL);
-                }
+	return;
         }
 
 
@@ -955,7 +960,7 @@ static void handle_connection_read(int sock, void *data){
                 }
 	else{
                 packet_age=(unsigned long)(current_time-packet_time);
-                if(packet_age > max_packet_age){
+                if(max_packet_age>0 && (packet_age>max_packet_age)){
                         syslog(LOG_ERR,"Dropping packet with stale timestamp - packet was %lu seconds old.",packet_age);
                         /*return;*/
 			close(sock);
@@ -1003,24 +1008,6 @@ static void handle_connection_read(int sock, void *data){
         }
 
 
-
-/* checks to see if a given host is allowed to talk to us */
-static int is_an_allowed_host(char *connecting_host){
-        char temp_buffer[MAX_INPUT_BUFFER];
-        char *temp_ptr;
-
-        strncpy(temp_buffer,allowed_hosts,sizeof(temp_buffer));
-        temp_buffer[sizeof(temp_buffer)-1]='\0';
-
-        for(temp_ptr=strtok(temp_buffer,",");temp_ptr!=NULL;temp_ptr=strtok(NULL,",")){
-                if(!strcmp(connecting_host,temp_ptr))
-                        return 1;
-                }
-
-        return 0;
-        }
-
-
 /* writes service/host check results to the Nagios command file */
 static int write_check_result(char *host_name, char *svc_description, int return_code, char *plugin_output, time_t check_time){
 
@@ -1169,6 +1156,7 @@ static int drop_privileges(char *user, char *group){
 				gid=(gid_t)(grp->gr_gid);
 			else
 				syslog(LOG_ERR,"Warning: Could not get group entry for '%s'",group);
+			endgrent();
 		        }
 
 		/* else we were passed the GID */
@@ -1194,6 +1182,7 @@ static int drop_privileges(char *user, char *group){
 				uid=(uid_t)(pw->pw_uid);
 			else
 				syslog(LOG_ERR,"Warning: Could not get passwd entry for '%s'",user);
+			endpwent();
 		        }
 
 		/* else we were passed the UID */
